from typing import List, Dict


URL_FILTER = "https://magma.esdm.go.id/api/v1/magma-var/filter"
URL_VOLCANO = "https://magma.esdm.go.id/api/v1/home/gunung-api"
URL_EVALUATION = "https://magma.esdm.go.id/api/v1/python/magma-var/evaluasi"

# Source Excel:
# https://1drv.ms/x/c/ef270b0f3925154b/EbVq29Kvwj9ArjOGIYM5vlIB8zFRIv5THo-r5aWcUuoVnQ?e=ekXAYA
EARTHQUAKE_TYPES: List[Dict[str, str]] = [
    {
        "code": "lts",
        "alias": "Eruption",
        "earthquake_id": "Letusan/Erupsi",
        "earthquake_en": "Eruption",
        "color": "#F44336",
        "column_name": "gempa.letusan_erupsi",
    },
    {
        "code": "apl",
        "alias": "PF - Fountain Collapse",
        "earthquake_id": "Awan Panas Letusan",
        "earthquake_en": "Fountain Collapse Pyroclastic Flow",
        "color": "#E91E63",
        "column_name": "gempa.awan_panas_letusan",
    },
    {
        "code": "apg",
        "alias": "PDC/PF",
        "earthquake_id": "Awan Panas Guguran",
        "earthquake_en": "Pyroclastic Density Current (PDC)",
        "color": "#673AB7",
        "column_name": "gempa.awan_panas_guguran",
    },
    {
        "code": "gug",
        "alias": "Rockfall",
        "earthquake_id": "Guguran",
        "earthquake_en": "Rockfall",
        "color": "#1976D2",
        "column_name": "gempa.guguran",
    },
    {
        "code": "hbs",
        "alias": "Degassing",
        "earthquake_id": "Hembusan",
        "earthquake_en": "Degassing",
        "color": "#3F51B5",
        "column_name": "gempa.hembusan",
    },
    {
        "code": "hrm",
        "alias": "Harmonic",
        "earthquake_id": "Harmonik",
        "earthquake_en": "Harmonic",
        "color": "#607D8B",
        "column_name": "gempa.harmonik",
    },
    {
        "code": "tre",
        "alias": "Non-Harmonic Tremor",
        "earthquake_id": "Tremor Non-Harmonik",
        "earthquake_en": "Non-Harmonic Tremor",
        "color": "#0D47A1",
        "column_name": "gempa.tremor_non-_harmonik",
    },
    {
        "code": "tor",
        "alias": "Tornillo",
        "earthquake_id": "Tornillo",
        "earthquake_en": "Tornillo",
        "color": "#03A9F4",
        "column_name": "gempa.tornillo",
    },
    {
        "code": "lof",
        "alias": "LF",
        "earthquake_id": "Low Frequency",
        "earthquake_en": "Low Frequency",
        "color": "#006064",
        "column_name": "gempa.low_frequency",
    },
    {
        "code": "hyb",
        "alias": "Hybrid/MP",
        "earthquake_id": "Hybrid/Fase Banyak",
        "earthquake_en": "Hybrid/Multi Phase",
        "color": "#009688",
        "column_name": "gempa.hybrid_fase_banyak",
    },
    {
        "code": "vtb",
        "alias": "VT-B",
        "earthquake_id": "Vulkanik Dangkal",
        "earthquake_en": "Shallow Volcanic-Tectonic (VT-B)",
        "color": "#8BC34A",
        "column_name": "gempa.vulkanik_dangkal",
    },
    {
        "code": "vta",
        "alias": "VT-A",
        "earthquake_id": "Vulkanik Dalam",
        "earthquake_en": "Deep Volcanic-Tectonic (VT-A)",
        "color": "#33691E",
        "column_name": "gempa.vulkanik_dalam",
    },
    {
        "code": "vlp",
        "alias": "VLP",
        "earthquake_id": "Very Long Period",
        "earthquake_en": "Very Long Period",
        "color": "#827717",
        "column_name": "gempa.very_long_period",
    },
    {
        "code": "tel",
        "alias": "Local Tectonic",
        "earthquake_id": "Tektonik Lokal",
        "earthquake_en": "Local Tectonic",
        "color": "#F57F17",
        "column_name": "gempa.tektonik_lokal",
    },
    {
        "code": "trs",
        "alias": "Felt Earthquake",
        "earthquake_id": "Terasa",
        "earthquake_en": "Felt Earthquake",
        "color": "#FFCA28",
        "column_name": "gempa.terasa",
    },
    {
        "code": "tej",
        "alias": "Teleseismic",
        "earthquake_id": "Tektonik Jauh",
        "earthquake_en": "Teleseismic",
        "color": "#FFA726",
        "column_name": "gempa.tektonik_jauh",
    },
    {
        "code": "dev",
        "alias": "Double Event",
        "earthquake_id": "Double Event",
        "earthquake_en": "Double Event",
        "color": "#FF5722",
        "column_name": "gempa.double_event",
    },
    {
        "code": "gtb",
        "alias": "Lahar",
        "earthquake_id": "Getaran Banjir",
        "earthquake_en": "Lahar",
        "color": "#795548",
        "column_name": "gempa.getaran_banjir",
    },
    {
        "code": "dpt",
        "alias": "Deep Tremor",
        "earthquake_id": "Deep Tremor",
        "earthquake_en": "Deep Tremor",
        "color": "#333333",
        "column_name": "gempa.deep_tremor",
    },
    {
        "code": "mtr",
        "alias": "Tremor",
        "earthquake_id": "Tremor Menerus",
        "earthquake_en": "Tremor (continuous)",
        "color": "#9E9E9E",
        "column_name": "gempa.tremor_menerus",
    },
]
